﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.ComponentModel.Composition;
using DotSpatial.Controls;
using DotSpatial.Topology;
using DotSpatial.Serialization;
using DotSpatial.Data;
using DotSpatial.Symbology;

namespace MES_Wind
{
    public partial class frmMain : Form
    {
        [Export("Shell", typeof(ContainerControl))]
        private static ContainerControl Shell;
        #region "Control variables"
         double distThreshold = 500;
        #endregion

        public frmMain()
        {

            InitializeComponent();
            if (DesignMode) return;
            Shell = this;
            appManager1.LoadExtensions();
        }
        #region
        /// <summary>
        /// This function is used to check if the segment of powerline is broken. 
        /// Based on the given line segment's start and endpoint, it divides the line on
        /// suitable subsegments defined by threshold length - dThreshold
        /// </summary>
        /// <param name="startX">Line segement's start X point</param>
        /// <param name="startY">Line segement's start Y point</param>
        /// <param name="endX">Line segement's end X point</param>
        /// <param name="endY">Line segement's end Y point</param>
        /// <param name="dThreshold">Line segment length threshold</param>
        /// <param name="Uwind_raster">Raster Layer for progmostic wind X coord</param>
        /// <param name="Vwind_raster">Raster Layer for progmostic wind Y coord</param>
        /// <returns>List of booleans with coordinates if any of them are true, line is broken</returns>
        /// <remarks></remarks>
#endregion
        List<PRMLibrary.Coordinate> DotspLinestringToPrm(LineString dtlinestr)
        {
            List<PRMLibrary.Coordinate> prmlinestring = new List<PRMLibrary.Coordinate>();
            IList<Coordinate> linepoints = dtlinestr.Coordinates;
            foreach(Coordinate linepoint in linepoints)
            {
                PRMLibrary.Coordinate point = new PRMLibrary.Coordinate(linepoint.X, linepoint.Y);
                prmlinestring.Add(point);
            }
            return prmlinestring;
        }
        PRMLibrary.Coordinate DotspPointToPRM(IPoint ftpoint)
        {
            PRMLibrary.Coordinate prpoint = new PRMLibrary.Coordinate();
            prpoint.X = ftpoint.X;
            prpoint.Y = ftpoint.Y;
            return prpoint;
        }
        
        public IFeatureSet brokenpoints(List<CheckPoint> chklist)
        {
            IFeatureSet points = new FeatureSet(FeatureType.Point);
            Coordinate chkcords = new Coordinate();
            return points;
        }

        private void bntLoadWindX_Click(object sender, EventArgs e)
        {
            map1.AddLayer();
            map1.ZoomToMaxExtent();
        }

        private void btnLoadWindY_Click(object sender, EventArgs e)
        {
            string curDir = Environment.CurrentDirectory;
            string path_to_tests = "\\MES_test_new\\";
            string file_path = curDir + path_to_tests;

            map1.AddLayer(file_path+"u_test.asc");
            map1.AddLayer(file_path + "v_test.asc");
            map1.AddLayer(file_path + "clim5_test.asc");
            map1.AddLayer(file_path + "clim10_test.asc");
            map1.AddLayer(file_path + "clim15_test.asc");
            map1.AddLayer(file_path + "powerlines.shp");
            map1.AddLayer(file_path + "powerstations.shp");
            map1.ZoomToMaxExtent();
        }

        private void btnViewAttributeTable_Click(object sender, EventArgs e)
        {
            //Declare a datatable
            DataTable dt = null;

            if (map1.GetLineLayers().Count() > 0)
            {
                MapLineLayer pwlLayer = default(MapLineLayer);

                pwlLayer = (MapLineLayer)map1.GetLineLayers()[0];

                if (pwlLayer == null)
                {
                    MessageBox.Show("The layer is not a line layer.");
                }
                else
                {
                    //Get the shapefile's attribute table to our datatable dt
                    dt = pwlLayer.DataSet.DataTable;

                    //Set the datagridview datasource from datatable dt
                    dgvAttributeTable.DataSource = dt;
                }
            }
            else
            {
                MessageBox.Show("Please add a layer to the map.");
            }

        

    }

        private void btnCalcStress_Click(object sender, EventArgs e)
        {
            try
            {
                //extract prognostic u layer
                IMapRasterLayer u_rasterLayer = default(IMapRasterLayer);
                IMapRasterLayer v_rasterLayer = default(IMapRasterLayer);
                IMapRasterLayer clim5_rasterLayer = default(IMapRasterLayer);
                IMapRasterLayer clim10_rasterLayer = default(IMapRasterLayer);
                IMapRasterLayer clim15_rasterLayer = default(IMapRasterLayer);

                if (map1.GetRasterLayers().Count() == 1)
                {
                    MessageBox.Show("Please add a raster layer");
                    return;
                }
                //this makes raster layers list in map1 ordered!
                u_rasterLayer = map1.GetRasterLayers()[0];
                v_rasterLayer = map1.GetRasterLayers()[1];
                clim5_rasterLayer = map1.GetRasterLayers()[2];
                clim10_rasterLayer = map1.GetRasterLayers()[3];
                clim15_rasterLayer = map1.GetRasterLayers()[4];

                //get the powerline  line layer
                IMapLineLayer pwlLayer = default(IMapLineLayer);
                if (map1.GetLineLayers().Count() == 0)
                {
                    MessageBox.Show("Please add powerline shapefile");
                    return;
                }
                pwlLayer = map1.GetLineLayers()[0];
                //copy line layer FeatureSet
                IFeatureSet pwlineSet = pwlLayer.DataSet;
                //get the powerstations layer 
                IMapPointLayer pwstLayer = default(IMapPointLayer);
                pwstLayer = map1.GetPointLayers()[0];
                // copy point layer FeatureSet
                IFeatureSet pwpointsSet = pwstLayer.DataSet;
                //Start to cast raster to PRM classes
                // prognostic wind massives first
                List<List<PRMLibrary.PrognosticCell>> prog_wind = new List<List<PRMLibrary.PrognosticCell>>();

                PRMLibrary.Coordinate dummy_coord = new PRMLibrary.Coordinate();
                Coordinate dummy_rcoord = new Coordinate();
                PRMLibrary.PrognosticCell dummy_prognostic = new PRMLibrary.PrognosticCell(dummy_coord, 0, 0);
                int rcount_prog = u_rasterLayer.DataSet.NumRows;
                int ccount_prog = u_rasterLayer.DataSet.NumColumns;
                for (int i =0; i< ccount_prog; i++)
                {
                    List<PRMLibrary.PrognosticCell> prog_wind_row = new List<PRMLibrary.PrognosticCell>();
                    for (int j =0; j< rcount_prog; j++ )
                    {
                        dummy_prognostic.velocityX = u_rasterLayer.DataSet.Value[j, i];
                        dummy_prognostic.velocityY = v_rasterLayer.DataSet.Value[j, i];
                        dummy_rcoord = u_rasterLayer.Bounds.CellCenter_ToProj(j,i);
                        dummy_coord.X = dummy_rcoord.X;
                        dummy_coord.Y = dummy_rcoord.Y;
                        dummy_prognostic.coords = dummy_coord;
                        prog_wind_row.Add(dummy_prognostic);                        
                    }
                    prog_wind.Add(prog_wind_row);
                    //prog_wind_row.Clear();
                }
                //Get cell info and Affine transform coefficients from prognostic wind rasters
                PRMLibrary.CellSize prog_cell = new PRMLibrary.CellSize(u_rasterLayer.DataSet.CellWidth, u_rasterLayer.DataSet.CellHeight);
                double[] prog_aff = u_rasterLayer.Bounds.AffineCoefficients;
                //Now we create climate raster class
                List<List<PRMLibrary.ClimateCell>> clim_wind = new List<List<PRMLibrary.ClimateCell>>();
                PRMLibrary.ClimateCell dummy_clim = new PRMLibrary.ClimateCell(dummy_coord, 0, 0, 0);
                int rcount_clim = clim5_rasterLayer.DataSet.NumRows;
                int ccount_clim = clim5_rasterLayer.DataSet.NumColumns;
                for (int i = 0; i < ccount_clim; i++)
                {
                    List<PRMLibrary.ClimateCell> clim_wind_row = new List<PRMLibrary.ClimateCell>();
                    for (int j = 0; j < rcount_clim; j++)
                    {
                        dummy_clim.wind5 = clim5_rasterLayer.DataSet.Value[j, i];
                        dummy_clim.wind10 = clim10_rasterLayer.DataSet.Value[j, i] ; // Important:  substract 3 to this to get powerlines broken
                        dummy_clim.wind15 = clim15_rasterLayer.DataSet.Value[j, i];
                        dummy_rcoord = clim15_rasterLayer.Bounds.CellCenter_ToProj(j,i);
                        dummy_coord.X = dummy_rcoord.X;
                        dummy_coord.Y = dummy_rcoord.Y;
                        dummy_clim.coords = dummy_coord;
                        clim_wind_row.Add(dummy_clim);
                    }
                    clim_wind.Add(clim_wind_row);
                    //clim_wind_row.Clear();
                }
                //Get cell info and affine transform coeff from climate rasters
                PRMLibrary.CellSize clim_cell = new PRMLibrary.CellSize(clim5_rasterLayer.DataSet.CellWidth, clim5_rasterLayer.DataSet.CellHeight);
                double[] clim_aff = clim5_rasterLayer.Bounds.AffineCoefficients;
                // create PRM_line list to pass to PRM_wind from loaded line layer
                List<PRMLibrary.Powerline> powerlinesToPRM = new List<PRMLibrary.Powerline>();
                foreach (IFeature feature in pwlineSet.Features)
                {
                    PRMLibrary.Powerline dummyline = new PRMLibrary.Powerline();
                    DataRow featureData = feature.DataRow;
                    dummyline.identifier = feature.Fid;
                    dummyline.year = int.Parse(featureData["Year"].ToString());
                    dummyline.height = double.Parse(featureData["HeightOffs"].ToString());
                    dummyline.power = int.Parse(featureData["Power"].ToString());
                    dummyline.pointFromID = int.Parse(featureData["PointFrom"].ToString());
                    dummyline.pointToID = int.Parse(featureData["PointTo"].ToString());
                    LineString ftline = feature.BasicGeometry as LineString;
                    dummyline.coords = DotspLinestringToPrm(ftline);
                    powerlinesToPRM.Add(dummyline);
                }
                //create PRM_station list to pass to PRM_wind from loaded point layer
                List<PRMLibrary.PowerStation> powerpointsToPRM = new List<PRMLibrary.PowerStation>();
                foreach (IFeature ftpoint in pwpointsSet.Features)
                {
                    PRMLibrary.PowerStation dummystation = new PRMLibrary.PowerStation();
                    DataRow featureData = ftpoint.DataRow;
                    dummystation.identifier = ftpoint.Fid;
                    dummystation.name = featureData["Name"].ToString();
                    dummystation.power = int.Parse(featureData["Power"].ToString());
                    int issr = int.Parse(featureData["IsSource"].ToString());
                    if (issr == 0)
                    {
                        dummystation.issource = false;
                    }
                    else
                    {
                        if (issr == 1)
                        {
                            dummystation.issource = true;
                        }
                        else
                        {
                            MessageBox.Show("Some entities in Column IsSource of powerstation datatable are not strict ones or zeros");
                        }
                    }
                    dummystation.type = featureData["Type"].ToString();
                    IPoint ftpt = ftpoint.BasicGeometry as IPoint;
                    dummystation.coords = DotspPointToPRM(ftpt);
                    powerpointsToPRM.Add(dummystation);

                }
                //Create a PRM_wind class and add all the properties from above
                PRMLibrary.Module prmwind = new PRMLibrary.Module();
                prmwind.input.powerLines = powerlinesToPRM;
                prmwind.input.powerStations = powerpointsToPRM;
                prmwind.input.prognosticCells = prog_wind;
                prmwind.input.prognosticCellSize = prog_cell;
                prmwind.input.prognosticAffineCoefficients = prog_aff;
                prmwind.input.climateCells = clim_wind;
                prmwind.input.climateCellSize = clim_cell;
                prmwind.input.climateAffineCoefficients = clim_aff;
                //Calculate which lines are broken
                List<PRMLibrary.Powerline> prmBrokenLines = prmwind.brokenPowerLinesAfterCheck();
                prmwind.CheckPower();
                // new FeatureSet for resulting broken powerlines
                //IFeatureSet brklineSet = new FeatureSet(FeatureType.Line);
                //DataTable dt = pwlineSet.DataTable;
                //IFeatureSet brk_info = new FeatureSet(FeatureType.Line);
                //brk_info = main_layer_function(pwlLayer, pwlineSet, u_rasterLayer, v_rasterLayer, clim15_rasterLayer, clim10_rasterLayer, clim5_rasterLayer);

                //IMapLineLayer brk_info_layer = (MapLineLayer)map1.Layers.Add(brk_info);
                //LineSymbolizer symbol = new LineSymbolizer(Color.Red, 3);
                //brk_info_layer.Symbolizer = symbol;
                //brk_info_layer.LegendText = "Broken powerlines";

            }
            catch (Exception ex)
            {
                MessageBox.Show("Something went wrong");
            }

        }
    }
    public class CheckPoint
    {
        public double X;
        public double Y;
        public bool Ifbroken;
    }

}
